/*
 WMMListBox.cpp
 Copyright (C) 1998-1999 R. Michael Sharkey (msharkey@canada.com)

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <WMMListBox.h>
#include <entryForm.h>
#include <propertiesForm.h>

#define inherited QListBox

WMMListBox::WMMListBox( QWidget *parent, const char *name, WFlags f )
    : inherited( parent, name, f )
{
    defaultIndex = 0;
    repeat = FALSE;
    launchDelay = timerVal = 0;
    timerID = (-1);
    
    commandLines.setAutoDelete( TRUE );
    allowList.setAutoDelete( TRUE );
    disallowList.setAutoDelete( TRUE );
    QObject::connect( this, SIGNAL(selected(int)), this, SLOT(activate()) );
    QObject::connect( this, SIGNAL(timeout()), this, SLOT(activate()) );
    QObject::connect( this, SIGNAL(highlighted(int)), this, SLOT(stopTimer()) );
    setFocusPolicy( StrongFocus );
    setEnabled( TRUE );
    setFocus();
}

WMMListBox::~WMMListBox()
{
}

void WMMListBox::activate()
{
    int status;
    pid_t pid;
    QString commandLine = commandLines.at( currentItem() )->data();
    
    stopTimer();
    pid = fork();
    
    switch ( pid ) {
    case 0:
        status = system( commandLine.data() ) >> 8;
        exit(status);
        break;
    case -1:
        QMessageBox::critical( this,
                               "Fork Failed",
                               "Failed to fork another process, probably low memory." );
        break;
    default:
        emit sigHide();
        while( waitpid( pid, &status, WNOHANG ) <= 0 ) {
            gApp->processEvents();
            sleep(1);
        }
        emit sigShow();
        status = status >> 8;
        if ( status != 0 )
            QMessageBox::critical( this,
                                   "Launch Failed",
                                   "Window manager terminated with an error\nor the window manager failed to launch." );
    }
    if ( repeat )
        initTimer();
}

bool WMMListBox::backup( QFile *file )
{
    char        buf[ 1024 ];
    QString     path = QString( file->name() ) + "~";
    QFile       backupFile( path );

    if ( !backupFile.open( IO_ReadWrite|IO_Truncate ) ) {
        goto fail;
    }
    file->reset();
    while ( !file->atEnd() ) {
        int len;
        if ( (len=file->readLine( buf, 1024 )) < 0 )
            goto fail;
        if ( backupFile.writeBlock( buf, len ) < 0 )
            goto fail;
    }
    backupFile.close();
    return TRUE;
fail:
    QMessageBox::critical( this,
                           "Backup Settings",
                           "Error encountered while making backup copy of settings file." );
    if ( backupFile.handle() >= 0 )
        backupFile.close();
    return FALSE;
    
}

bool WMMListBox::setAccessControl( void )
{
    int status=0;

    if ( allowList.count() > 0 || disallowList.count() > 0 ) {
        status=system( "xhost -" );
        if ( status != 0 ) {
            QMessageBox::warning( this,
                                  "Access Control",
                                  "An access control problem was encountered.\nMake sure \"xhost\" is installed and functioning." );
            return FALSE;
        }
    }

    for ( int index=0; index < allowList.count(); index++ ) {
        char buf[ 1024 ];
        sprintf( buf, "xhost +%s", allowList.at( index )->data() );
        gApp->processEvents();
        status=system( buf );
        if ( status != 0 ) {
            QString str = QString( "Attempt to grant display access to \"" ) +
                QString( *(allowList.at( index )) ) +
                QString( "\" failed." );
            QMessageBox::warning( this,
                           "Access Control",
                           str.data() );
        }
    }

    for ( int index=0; index < disallowList.count(); index++ ) {
        char buf[ 1024 ];
        sprintf( buf, "xhost -%s", disallowList.at( index )->data() );
        gApp->processEvents();
        system( buf );
        if ( status != 0 ) {
            QString str = QString( "Attempt to restrict display access from \"" ) +
                QString( *(disallowList.at( index )) ) +
                QString( "\" failed." );
            QMessageBox::warning( this,
                           "Access Control",
                           str.data() );
        }
    }

    return TRUE;
}

bool WMMListBox::saveProperties( QFile *file )
{
    QString propString;
    int index;

    // repeat
    propString.sprintf( "repeat = %d\n", repeat?1:0 );
    if ( file->writeBlock( propString.data(), propString.length() ) < 0 )
        goto fail;

    // launchDelay
    propString.sprintf( "launchDelay = %d\n", launchDelay );
    if ( file->writeBlock( propString.data(), propString.length() ) < 0 )
        goto fail;

    // allow list
    for ( index=0; index < allowList.count(); index++ ) {
        propString.sprintf( "allow = %s\n", allowList.at( index )->data() );
        if ( file->writeBlock( propString.data(), propString.length() ) < 0 )
            goto fail;
    }

    // disallow list
    for ( index=0; index < disallowList.count(); index++ ) {
        propString.sprintf( "disallow = %s\n", disallowList.at( index )->data() );
        if ( file->writeBlock( propString.data(), propString.length() ) < 0 )
            goto fail;
    }

    return TRUE;
fail:
    return FALSE;
}

bool WMMListBox::saveEntries( QFile *file )
{
    for ( int index=0; index < count(); index++ ) {
        QString outText;
        QString titleText( text( index ) );
        QString commandLineText( commandLines.at( index )->data() );
        titleText = titleText.stripWhiteSpace();
        commandLineText = commandLineText.stripWhiteSpace();
        outText.sprintf( "%s%s : %s\n", (defaultIndex==index)?"*":"", titleText.data(), commandLineText.data() );
        if ( file->writeBlock( outText.data(), outText.length() ) < 0 )
            goto fail;
    }
    return TRUE;
fail:
    return FALSE;
}

bool WMMListBox::save( void )
{
    QFile       file;
    QTextStream stream( &file );
    QString     path( getenv( "HOME" ) );

    if ( path.isEmpty() )
        goto fail;

    path += "/";
    path += ".xwmmrc";
    file.setName( path );

    // make a backup copy first...
    if ( file.exists( file.name() ) ) {
        if ( !file.open( IO_ReadOnly ) )
            goto fail;
        if ( !backup( &file ) )
            goto fail;
        file.close();
    }

    // ok, now rewrite the file...
    if ( !file.open( IO_ReadWrite|IO_Truncate ) )
        goto fail;
    if ( !saveProperties( &file ) )
        goto fail;
    if ( !saveEntries( &file ) )
        goto fail;
    file.close();
    return TRUE;
fail:
    QMessageBox::critical( this,
                           "Save Settings",
                           "Error encountered while saving settings file." );
    if ( file.handle() >= 0 )
        file.close();
    return FALSE;

}

bool WMMListBox::readStream( QTextStream *stream )
{
    clear();
    commandLines.clear();
    allowList.clear();
    disallowList.clear();
    
    while ( !stream->eof() ) {
        QString line = stream->readLine();
        // parse the line; <button name>:<command line>
        line.simplifyWhiteSpace();
        if ( line[0] != '#' ) { // ignore comment line
            int sepPos;
            if ( ( sepPos = line.find( ':', 0, FALSE ) ) > 0 ) { // list entry line
                QString itemText( line.mid( 0, sepPos ) );
                QString *commandLine = new QString( line.mid( sepPos+1, line.length()-sepPos ) );
                itemText.simplifyWhiteSpace();
                commandLine->simplifyWhiteSpace();
                commandLines.append( commandLine );
                if ( itemText[0] == '*' ) {
                    itemText = itemText.right( itemText.length()-1 );
                    insertItem( itemText );
                    setCurrentItem( (defaultIndex=(count()-1)) );
                } else {
                    insertItem( itemText );
                }
            } else if ( ( sepPos = line.find( '=', 0, FALSE ) ) > 0 ) { // variable assignment
                QString variable = line.mid( 0, sepPos );
                QString arg = line.mid( sepPos+1, line.length()-sepPos );
                int intArg = atoi( arg.data() );
                if ( strncmp( variable,"launchDelay",11) == 0 )
                    launchDelay = intArg;
                else if ( strncmp( variable,"repeat",6) == 0  )
                    repeat = intArg!=0 ? TRUE : FALSE;
                else if ( strncmp( variable,"allow",5) == 0 ) {
                    QString *allow = new QString( arg.simplifyWhiteSpace() );
                    allowList.append( allow );
                }
                else if ( strncmp( variable,"disallow",8) == 0 ) {
                    QString *disallow = new QString( arg.simplifyWhiteSpace() );
                    disallowList.append( disallow );
                }
            }
        }
    }

//    if ( count() == 0 )
//        goto fail;

    if ( currentItem() < 0 && count() > 0 )
        setCurrentItem( 0 );

    return TRUE;
}

bool WMMListBox::load( void )
{
    QFile       file;
    QTextStream stream( &file );
    QString     path( getenv( "HOME" ) );
    
    stopTimer();
    if ( path.isEmpty() )
        goto fail;

    path += "/";
    path += ".xwmmrc";
    file.setName( path );
    if ( !file.exists() ) {
        QString sysPath=getenv("SYSXWMMRC")!=NULL?getenv("SYSXWMMRC"):SYSXWMMRC;
        file.setName( sysPath );
        if ( file.open( IO_ReadOnly ) ) {
            readStream( &stream );
            file.close();
        }
        save();
        file.setName( path );
    }
    if ( !file.open( IO_ReadOnly ) ) {
        goto fail;
    }

    readStream( &stream );

    if ( file.handle() >= 0 )
        file.close();

    setAccessControl();

    initTimer();
    return TRUE;
fail:
    if ( file.handle() >= 0 )
        file.close();
    QMessageBox::critical( this,
                           "Load Settings",
                           "Error encountered while loading settings file." );
    return FALSE;
}

void WMMListBox::newEntry()
{
    // interactive: create a new entry
    entryForm form;
    if ( form.exec() == QDialog::Accepted ) {
        char *pTitle = form.getTitleText();
        QString *commandLine = new QString( form.getCommandLineText() );
        bool isDefault = form.getDefault();
        //debug("newEntry(%s%s : %s)", isDefault?"*":"", pTitle, commandLine->data() );
        if ( isDefault )
            defaultIndex = count();
        insertItem( pTitle );
        commandLines.append( commandLine );
        save();
        load();
    }
}

void WMMListBox::deleteEntry()
{
    // interactive: delete an entry
    int index = currentItem();
    if ( index >= 0 ) {
        QString msgText;
        msgText.sprintf( "Delete \"%s\"?", text( index ) );
        if ( QMessageBox::warning( this, "Delete Entry", msgText.data(), "OK", "Cancel", 0 ) == 0 ) {
            //debug("deleteEntry=%d",index);
            if ( defaultIndex == index ) {
                defaultIndex=0;
            } else if ( defaultIndex > index  ) {
                defaultIndex-=1;
            }
            commandLines.remove( index );
            removeItem( index );
            save();
            load();
        }
    } 
}

void WMMListBox::reviseEntry()
{
    // interactive: revise an entry
    int index = currentItem();
    if ( index >= 0 ) {
        entryForm form;
        form.setTitleText( text( index ) );
        form.setCommandLineText( commandLines.at( index )->data() );
        form.setDefault( (defaultIndex==index)?TRUE:FALSE );
        if ( form.exec() == QDialog::Accepted ) {
            char *pTitle = form.getTitleText();
            QString *commandLine = new QString( form.getCommandLineText() );
            bool isDefault = form.getDefault();
            //debug("reviseEntry(%s%s : %s)", isDefault?"*":"", pTitle, commandLine->data() );
            if ( isDefault )
                defaultIndex = index;
            changeItem( pTitle, index );
            commandLines.remove( index );
            commandLines.insert( index, commandLine );
            save();
            load();
        }
    }
}

void WMMListBox::reviseProperties()
{
    // interactive: revise properties
    propertiesForm form;
    form.setDelay( launchDelay );
    form.setRepeat( repeat );
    form.setAllow( &allowList );
    form.setDisallow( &disallowList );
    if ( form.exec() == QDialog::Accepted ) {
        launchDelay = form.getDelay();
        repeat = form.getRepeat();
        form.getAllow();
        form.getDisallow();
        //debug("reviseProperties(delay=%d,repeat=%d)",launchDelay,repeat);
        save();
        load();
    }
}

void WMMListBox::initTimer( void )
{
    timerVal = launchDelay;
    if ( !activeTimer() && timerVal > 0 )
        timerID = startTimer( 1000 );
    emit tick( timerVal );
}

void WMMListBox::stopTimer( void )
{
    if ( activeTimer() ) {
        killTimer( timerID );
        timerID = (-1);
    }
    timerVal=0;
    emit tick( 0 );
}

bool WMMListBox::activeTimer( void )
{
    return ( timerID != (-1) );
}

void WMMListBox::timerEvent( QTimerEvent *e )
{
    if ( e->timerId() == timerID ) {
        // should get here once per second
        if ( timerVal == 0 ) {
            // ok, time to launch
            emit tick( 0 );
            emit timeout();
        } else {
            emit tick( --timerVal );
        }
    }
}

void WMMListBox::mousePressEvent( QMouseEvent *e )
{
    stopTimer();
    inherited::mousePressEvent( e );
}

void WMMListBox::mouseReleaseEvent( QMouseEvent *e )
{
    inherited::mouseReleaseEvent( e );
}

void WMMListBox::mouseDoubleClickEvent( QMouseEvent *e )
{
    inherited::mouseDoubleClickEvent( e );
}

void WMMListBox::keyPressEvent( QKeyEvent *e )
{
    stopTimer();
    inherited::keyPressEvent( e );
}
