/*
 WindowManagerManager.cpp
 Copyright (C) 1998-1999 R. Michael Sharkey (msharkey@canada.com)

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "WindowManagerManager.h"
#include "aboutDialog.h"
#include "xwmm.xpm"
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>

#define inherited QWidget


#define max(a,b) ( a>b ? a : b )

WindowManagerManager::WindowManagerManager( QWidget *parent, const char *name, WFlags f ) :
    inherited( parent, name, f )
{
    setFocusPolicy( StrongFocus );
    setEnabled( TRUE );
    setFocus();

    QPixmap pixmap( (const char **)xwmm );
    int runHeight=0;

    setIcon( pixmap );


    QPopupMenu *file = new QPopupMenu();
    CHECK_PTR( file );
    file->insertItem( "&Quit", gApp, SLOT(quit()), CTRL+Key_Q );

    QPopupMenu *edit = new QPopupMenu();
    CHECK_PTR( edit );
    edit->insertItem( "&New", this, SLOT(newEntry()), CTRL+Key_N );
    edit->insertItem( "&Delete", this, SLOT(deleteEntry()), CTRL+Key_X );
    edit->insertItem( "&Revise", this, SLOT(reviseEntry()), CTRL+Key_R );
    edit->insertSeparator();
    edit->insertItem( "Prop&erties", this, SLOT(reviseProperties()), CTRL+Key_E );
    
    QPopupMenu *help = new QPopupMenu;
    CHECK_PTR( help );
    help->insertItem( "&About", this, SLOT(aboutWMM()), CTRL+Key_H );

    pMenuBar = new QMenuBar( this );
    CHECK_PTR( pMenuBar );
    pMenuBar->insertItem( "&File", file );
    pMenuBar->insertItem( "&Edit", edit );
    pMenuBar->insertSeparator();
    pMenuBar->insertItem( "&Help", help );
    pMenuBar->setSeparator( QMenuBar::InWindowsStyle );
    runHeight += pMenuBar->height();
    
    pIcon = new QLabel( this );
    CHECK_PTR( pIcon );
    pIcon->setAutoResize( TRUE );
    pIcon->setPixmap( pixmap );
    pIcon->move( 0, runHeight );

    pTitle = new QLabel( WMM_VERSION, this );
    CHECK_PTR( pTitle );
    pTitle->move( pIcon->width()+4, runHeight );
    pTitle->show();
    runHeight += pIcon->height();

    pListBox = new WMMListBox( this );
    CHECK_PTR( pListBox );
    QObject::connect( pListBox, SIGNAL(sigHide()), this, SLOT(hide()) );
    QObject::connect( pListBox, SIGNAL(sigShow()), this, SLOT(show()) );
    QObject::connect( pListBox, SIGNAL(tick(int)), this, SLOT(tick(int)) );
    pListBox->setEnabled( TRUE );
    pListBox->setFocusPolicy( QWidget::StrongFocus );
    pListBox->move( 0, runHeight );
    pListBox->resize( pListBox->width() * 2, pListBox->height() * 4 );
    pListBox->show();
    pListBox->load();
    runHeight += pListBox->height();
    
    pCloseButton = new QPushButton( "Close", this );
    CHECK_PTR( pCloseButton );
    QObject::connect( pCloseButton, SIGNAL(clicked()), gApp, SLOT(quit()) );
    pCloseButton->setEnabled( TRUE );
    pCloseButton->setFocusPolicy( QWidget::StrongFocus );
    pCloseButton->move( 0, runHeight );
    pCloseButton->resize( pListBox->width(), pCloseButton->height() );
    pCloseButton->show();
    runHeight += pCloseButton->height();

    resize( max( pTitle->width(), pListBox->width() ), runHeight );
    setMinimumSize( size() );
    setMaximumSize( size() );
    move( QApplication::desktop()->width()/2 - width()/2, QApplication::desktop()->height()/2 - height()/2 );
    show();

    pListBox->setFocusPolicy( StrongFocus );
    pListBox->setFocus();
    XSetInputFocus( qt_xdisplay(), pListBox->winId(), RevertToParent, CurrentTime );
    
    QObject::connect( pMenuBar, SIGNAL( highlighted(int) ), this, SLOT( menuBarActive(int) ) );
}

WindowManagerManager::~WindowManagerManager()
{
    delete pMenuBar;
    delete pIcon;
    delete pTitle;
    delete pListBox;
    delete pCloseButton;
}

void WindowManagerManager::newEntry()
{
    pListBox->newEntry();
}

void WindowManagerManager::deleteEntry()
{
    pListBox->deleteEntry();
}

void WindowManagerManager::reviseEntry()
{
    pListBox->reviseEntry();
}

void WindowManagerManager::reviseProperties()
{
    pListBox->reviseProperties();
}

void WindowManagerManager::aboutWMM()
{
    aboutDialog dialog( "About WMM",
                        "X Window Manager Manager\n" \
                        WMM_VERSION \
                        "\n" \
                        "Copyright (C) 1998,1999  R. Michael Sharkey (msharkey@canada.com)\n" \
                        "\n" \
                        "This program is free software; you can redistribute it and/or modify\n" \
                        "it under the terms of the GNU General Public License as published by\n" \
                        "the Free Software Foundation; either version 2 of the License, or\n" \
                        "(at your option) any later version.\n" \
                        "\n" \
                        "This program is distributed in the hope that it will be useful,\n" \
                        "but WITHOUT ANY WARRANTY; without even the implied warranty of\n" \
                        "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n" \
                        "GNU General Public License for more details.\n" \
                        "\n" \
                        "You should have received a copy of the GNU General Public License\n" \
                        "along with this program; if not, write to the Free Software\n" \
                        "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n",
                        QMessageBox::Information,
                        QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape,
                        0,
                        0
                      );
    dialog.exec();
}

void WindowManagerManager::menuBarActive( int )
{
    pListBox->stopTimer();
}

void WindowManagerManager::tick( int timerVal)
{
    QPainter paint;
    QString countStr;
    countStr.sprintf( "[%2d]", timerVal );
    if ( !countStr.isNull() ) {
        QRect textRect = fontMetrics().boundingRect( countStr );
        textRect.moveTopRight( QPoint( width()-10, pMenuBar->height()+10 ) );
        textRect.setWidth( textRect.width()+2 );
        textRect.setHeight( textRect.height()+2 );
        paint.begin( this );
        paint.eraseRect( textRect );
        if ( timerVal > 0 )
            paint.drawText( textRect, AlignVCenter|AlignHCenter, countStr );
        paint.end();
    }
}

void WindowManagerManager::enterEvent( QEvent *e )
{
    inherited::enterEvent( e );
    XSetInputFocus( qt_xdisplay(), pListBox->winId(), RevertToParent, CurrentTime );
}

void WindowManagerManager::leaveEvent( QEvent *e )
{
    inherited::leaveEvent( e );
}

void WindowManagerManager::paintEvent( QPaintEvent *e )
{
    inherited::paintEvent( e );
    QPainter paint( this );
    QRect bounds = rect();
    qDrawShadePanel( &paint,
                     bounds.x(), bounds.y(), bounds.width(), bounds.height(),
                     colorGroup() );
    QApplication::desktop()->setBackgroundColor( QColor( 0x45,0x82, 0xb6 ) );
}
